import json
from pyecharts import Line, Page

from stagesep2.logger import logger
from stagesep2.config import NormalConfig


class ReportPainter(object):
    TAG = 'Painter'

    @classmethod
    def draw_with_file(cls, file, dst):
        with open(file, encoding=NormalConfig.CHARSET) as f:
            cls.draw_with_dict(json.load(f), dst)

    @classmethod
    def draw_with_json(cls, json_str, dst):
        content = json.loads(json_str)
        cls.draw_with_dict(content, dst)

    @classmethod
    def draw_with_dict(cls, content, dst):
        # base page
        page = Page()

        # time line
        time_list = [each['current_time'] for each in content]

        # trend
        if 'trend' in content[0]:
            trend_list = [each['trend'] for each in content]
            trend_line = cls.build_trend_line(time_list, trend_list)
            page.add(trend_line)

        # match template
        if 'match_template' in content[0]:
            match_template_list = [each['match_template'] for each in content]
            match_template_line = cls.build_match_template_line(time_list, match_template_list)
            page.add(match_template_line)

        page.render(dst)
        logger.info(cls.TAG, msg='report built finished: "{}"'.format(dst))

    @classmethod
    def build_trend_line(cls, time_list, trend_list):
        trend_line = Line('trend')
        for each_attr in ['previous', 'first', 'last']:
            each_list = [i[each_attr] for i in trend_list]
            trend_line.add(
                each_attr,
                time_list,
                each_list,
                yaxis_min='dataMin',
                is_more_utils=True,
            )
        return trend_line

    @classmethod
    def build_match_template_line(cls, time_list, match_template_list):
        # 直接用float作为x轴会出现 echarts 兼容问题
        time_list = [str(each) for each in time_list]

        match_template_dict = dict()
        for each_template in match_template_list:
            for each_name, each_value in each_template.items():
                if each_name in match_template_dict:
                    match_template_dict[each_name].append(each_value)
                else:
                    match_template_dict[each_name] = [each_value]

        match_template_line = Line('match_template')
        for each_name, each_value_list in match_template_dict.items():
            for each_attr in ['min', 'max']:
                each_list = [i[each_attr] for i in each_value_list]
                match_template_line.add(
                    '{}_{}'.format(each_name, each_attr),
                    x_axis=time_list,
                    y_axis=each_list,
                    is_more_utils=True,
                )
        return match_template_line



"""
分析器

- OCR
- 特征识别
"""
import cv2

from stagesep2.loader import VideoManager, frame_prepare
from stagesep2.config import NormalConfig
from stagesep2.logger import logger
from stagesep2.reporter import ResultReporter, ResultRow
from stagesep2.utils import *


class AnalysisRunner(object):
    """
    主要逻辑

    - 从VideoManager中导入视频对象
    - 从config中读取需要使用的Analyser
    - 遍历视频列表
        - 切割视频，遍历帧
            - 用不同的Analyser分析帧
            - 记录结果
    - 将结果传递给reporter进行处理
    """
    TAG = 'AnalyserRunner'

    @classmethod
    def run(cls):
        analyser_list = check_analyser(NormalConfig.ANALYSER_LIST)
        video_dict = VideoManager.video_dict
        logger.info(cls.TAG, analyser=analyser_list, video=video_dict)

        result_reporter_list = list()
        for each_video_name, each_ssv in video_dict.items():
            result_reporter = ResultReporter(each_video_name)
            cls.analyse_video(each_ssv, analyser_list, result_reporter)
            result_reporter_list.append(result_reporter)

        # export result
        if len(result_reporter_list) <= 1:
            return result_reporter_list[0]
        return result_reporter_list

    @classmethod
    def analyse_video(cls, ssv_video, analyser_list, result_reporter):
        """ analyse ssv video """
        with video_capture(ssv_video) as each_video:
            ret, frame = each_video.read()
            while ret:
                if not ret:
                    # end of video
                    break

                # prepare frame
                frame = frame_prepare(frame)
                # rotate
                frame = rotate_pic(frame, ssv_video.rotate)

                # current status
                cur_frame_count = int(each_video.get(cv2.CAP_PROP_POS_FRAMES))
                cur_second = each_video.get(cv2.CAP_PROP_POS_MSEC) / 1000

                # new row of result
                new_row = ResultRow(
                    result_reporter.result_id,
                    ssv_video.video_path,
                    cur_frame_count,
                    cur_second,
                )

                for each_analyser in analyser_list:
                    result = each_analyser.run(frame, ssv_video)
                    new_row.add_analyser_result(each_analyser.name, result)

                logger.info(cls.TAG, msg='analysing', **new_row.__dict__)
                result_reporter.add_row(new_row)

                # read new frame
                ret, frame = each_video.read()

        # clean analyser
        for each in analyser_list:
            each.clean()



from skimage.measure import compare_ssim

from stagesep2.analyser.base import BaseAnalyser


class TrendAnalyser(BaseAnalyser):
    """ trend analyser """
    name = 'trend'
    previous_frame = None

    @classmethod
    def run(cls, frame, ssv):
        """
        return a dict, eg: {'previous': 1.0, 'first': 0.9, 'last': 0.2}

        :param frame:
        :param ssv:
        :return:
        """
        # init previous frame
        if cls.previous_frame is None:
            cls.previous_frame = frame

        # calculate sim
        previous_sim = compare_ssim(cls.previous_frame, frame)
        first_sim = compare_ssim(ssv.first_frame, frame)
        last_sim = compare_ssim(ssv.last_frame, frame)

        # update previous frame
        cls.previous_frame = frame

        result_dict = {
            'previous': previous_sim,
            'first': first_sim,
            'last': last_sim,
        }
        return result_dict

    @classmethod
    def clean(cls, *args, **kwargs):
        cls.previous_frame = None



"""
分析配置存放在此处
支持读取外部json配置
"""
import os
import cv2


class OCRConfig(object):
    lang = 'eng'


class MatchTemplateConfig(object):
    cv_method = cv2.TM_CCOEFF_NORMED


class NormalConfig(object):
    # project root path
    PROJECT_PATH = os.path.dirname(os.path.dirname(__file__))

    # default analyser list (will run)
    ANALYSER_LIST = ['ocr', 'match_template', 'trend']

    # default encoding
    CHARSET = 'utf-8-sig'



from stagesep2.analyser.base import BaseAnalyser
from findit import FindIt


class MatchTemplateAnalyser(BaseAnalyser):
    """ match-template analyser """
    name = 'match_template'

    @classmethod
    def run(cls, frame, ssv):
        result_dict = dict()
        template_pic_dict = ssv.template_manager.get_dict()

        for each_pic_name, each_pic in template_pic_dict.items():
            fi = FindIt()
            fi.template[each_pic.pic_path] = each_pic.cv_object
            match_result = fi.find(target_cv_object=frame)['data'][0]
            max_val, min_val = match_result['max_val'], match_result['min_val']

            result_dict[each_pic_name] = {
                'min': min_val,
                'max': max_val,
            }
        return result_dict



class BaseAnalyser(object):
    name = ''

    @classmethod
    def run(cls, *args, **kwargs):
        """
        normally,

        1. frame (opencv image object)
        2. ssv (provide necessary info, eg: template pictures)

        :param args:
        :param kwargs:
        :return:
        """
        return ''

    @classmethod
    def clean(cls, *args, **kwargs):
        """
        clean Analyser config after usage

        :param args:
        :param kwargs:
        :return:
        """



from setuptools import setup, find_packages


setup(
    name='stagesep2',
    version='0.2.1',
    description='Analyse, and convert video into useful data.',
    author='williamfzc',
    author_email='fengzc@vip.qq.com',
    url='https://github.com/williamfzc/stagesep2',
    packages=find_packages(),
    install_requires=[
        'opencv-python',
        'structlog',
        'numpy',
        'jieba',
        'scikit-image',
        'pyecharts',
        'pyecharts_snapshot',
        'findit',
    ]
)
